@@ -17,12 +17,29 @@ log = logging.getLogger('py3fdfs')
 
 
 def get_tracker_conf(conf_path='client.conf'):
+    conf_path = os.path.expanduser(conf_path)
     cf = Fdfs_ConfigParser()
     tracker = {}
     try:
         cf.read(conf_path)
         timeout = cf.getint('__config__', 'connect_timeout')
         tracker_list = cf.get('__config__', 'tracker_server')
+        use_storage_id = cf.get('__config__', 'use_storage_id', fallback='false')
+        use_storage_id = use_storage_id == 'true'
+        tracker['use_storage_id'] = use_storage_id
+
+        if use_storage_id:
+            group_ids_conf_path = os.path.join(os.path.dirname(conf_path), 'storage_ids.conf')
+            groups = {}
+            with open(group_ids_conf_path) as fin:
+                for line in fin.readlines():
+                    if line.startswith('#') or 'group' not in line:
+                        continue
+                    gid, group_name, ips = line.split()
+                    ips = ips.encode()
+                    groups[group_name] = ips.split(b',')
+            tracker['groups'] = groups
+
         if isinstance(tracker_list, str):
             tracker_list = [tracker_list]
         tracker_ip_list = []
@@ -39,12 +56,12 @@ def get_tracker_conf(conf_path='client.conf'):
 
 
 class Fdfs_client(object):
-    '''
+    """
     Class Fdfs_client implemented Fastdfs client protol ver 3.08.
 
     It's useful upload, download, delete file to or from fdfs server, etc. It's uses
     connection pool to manage connection to server.
-    '''
+    """
 
     def __init__(self, trackers, poolclass=ConnectionPool):
         self.trackers = trackers
@@ -60,7 +77,7 @@ class Fdfs_client(object):
             pass
 
     def upload_by_filename(self, filename, meta_dict=None, file_crypt=None):
-        '''
+        """
         Upload a file to Storage server.
         arguments:
         @filename: string, name of file that will be uploaded
@@ -78,12 +95,13 @@ class Fdfs_client(object):
             'Uploaded size'   : upload_size,
             'Storage IP'      : storage_ip
         } if success else None
-        '''
+        """
         isfile, errmsg = fdfs_check_file(filename)
         if not isfile:
             raise DataError(errmsg + '(uploading)')
-        tc = Tracker_client(self.tracker_pool)
+        tc = Tracker_client(self.tracker_pool, self.trackers)
         store_serv = tc.tracker_query_storage_stor_without_group()
+
         store = Storage_client(store_serv.ip_addr, store_serv.port, self.timeout)
         return store.storage_upload_by_filename(tc, store_serv, filename, meta_dict, file_crypt)
 
@@ -97,7 +115,7 @@ class Fdfs_client(object):
         return store.storage_upload_by_file(tc, store_serv, filename, meta_dict)
 
     def upload_by_buffer(self, filebuffer, file_ext_name=None, meta_dict=None):
-        '''
+        """
         Upload a buffer to Storage server.
         arguments:
         @filebuffer: string, buffer
@@ -116,7 +134,7 @@ class Fdfs_client(object):
             'Uploaded size'   : upload_size,
             'Storage IP'      : storage_ip
         } if success else None
-        '''
+        """
         if not filebuffer:
             raise DataError('[-] Error: argument filebuffer can not be null.')
         tc = Tracker_client(self.tracker_pool)
@@ -125,7 +143,7 @@ class Fdfs_client(object):
         return store.storage_upload_by_buffer(tc, store_serv, filebuffer, file_ext_name, meta_dict)
 
     def upload_slave_by_filename(self, filename, remote_file_id, prefix_name, meta_dict=None):
-        '''
+        """
         Upload slave file to Storage server.
         arguments:
         @filename: string, local file name
@@ -144,7 +162,7 @@ class Fdfs_client(object):
             'Remote file id'  : remote_file_id,
             'Storage IP'      : storage_ip
         }
-        '''
+        """
         isfile, errmsg = fdfs_check_file(filename)
         if not isfile:
             raise DataError(errmsg + '(uploading slave)')
@@ -166,7 +184,7 @@ class Fdfs_client(object):
         return ret_dict
 
     def upload_slave_by_file(self, filename, remote_file_id, prefix_name, meta_dict=None):
-        '''
+        """
         Upload slave file to Storage server.
         arguments:
         @filename: string, local file name
@@ -185,7 +203,7 @@ class Fdfs_client(object):
             'Remote file id'  : remote_file_id,
             'Storage IP'      : storage_ip
         }
-        '''
+        """
         isfile, errmsg = fdfs_check_file(filename)
         if not isfile:
             raise DataError(errmsg + '(uploading slave)')
@@ -207,7 +225,7 @@ class Fdfs_client(object):
         return ret_dict
 
     def upload_slave_by_buffer(self, filebuffer, remote_file_id, meta_dict=None, file_ext_name=None):
-        '''
+        """
         Upload slave file by buffer
         arguments:
         @filebuffer: string
@@ -225,7 +243,7 @@ class Fdfs_client(object):
             'Remote file id'  : remote_file_id,
             'Storage IP'      : storage_ip
         }
-        '''
+        """
         if not filebuffer:
             raise DataError('[-] Error: argument filebuffer can not be null.')
         tmp = split_remote_fileid(remote_file_id)
@@ -239,7 +257,7 @@ class Fdfs_client(object):
                                                     file_ext_name)
 
     def upload_appender_by_filename(self, local_filename, meta_dict=None):
-        '''
+        """
         Upload an appender file by filename.
         arguments:
         @local_filename: string
@@ -257,7 +275,7 @@ class Fdfs_client(object):
             'Uploaded size'   : upload_size,
             'Storage IP'      : storage_ip
         } if success else None
-        '''
+        """
         isfile, errmsg = fdfs_check_file(local_filename)
         if not isfile:
             raise DataError(errmsg + '(uploading appender)')
@@ -267,7 +285,7 @@ class Fdfs_client(object):
         return store.storage_upload_appender_by_filename(tc, store_serv, local_filename, meta_dict)
 
     def upload_appender_by_file(self, local_filename, meta_dict=None):
-        '''
+        """
         Upload an appender file by file.
         arguments:
         @local_filename: string
@@ -285,7 +303,7 @@ class Fdfs_client(object):
             'Uploaded size'   : upload_size,
             'Storage IP'      : storage_ip
         } if success else None
-        '''
+        """
         isfile, errmsg = fdfs_check_file(local_filename)
         if not isfile:
             raise DataError(errmsg + '(uploading appender)')
@@ -295,7 +313,7 @@ class Fdfs_client(object):
         return store.storage_upload_appender_by_file(tc, store_serv, local_filename, meta_dict)
 
     def upload_appender_by_buffer(self, filebuffer, file_ext_name=None, meta_dict=None):
-        '''
+        """
         Upload a buffer to Storage server.
         arguments:
         @filebuffer: string
@@ -309,35 +327,35 @@ class Fdfs_client(object):
             'Uploaded size'   : upload_size,
             'Storage IP'      : storage_ip
         } if success else None
-        '''
+        """
         if not filebuffer:
             raise DataError('[-] Error: argument filebuffer can not be null.')
-        tc = Tracker_client(self.tracker_pool)
+        tc = Tracker_client(self.tracker_pool, self.trackers)
         store_serv = tc.tracker_query_storage_stor_without_group()
         store = Storage_client(store_serv.ip_addr, store_serv.port, self.timeout)
         return store.storage_upload_appender_by_buffer(tc, store_serv, filebuffer, meta_dict, file_ext_name)
 
     def delete_file(self, remote_file_id):
-        '''
+        """
         Delete a file from Storage server.
         arguments:
         @remote_file_id: string, file_id of file that is on storage server
         @return tuple ('Delete file successed.', remote_file_id, storage_ip)
-        '''
+        """
         tmp = split_remote_fileid(remote_file_id)
         if not tmp:
             raise DataError('[-] Error: remote_file_id is invalid.(in delete file)')
         group_name, remote_filename = tmp
-        tc = Tracker_client(self.tracker_pool)
+        tc = Tracker_client(self.tracker_pool, self.trackers)
         store_serv = tc.tracker_query_storage_update(group_name, remote_filename)
         store = Storage_client(store_serv.ip_addr, store_serv.port, self.timeout)
         return store.storage_delete_file(tc, store_serv, remote_filename)
 
     def download_to_file(self, local_filename, remote_file_id, offset=0, down_bytes=0, file_crypt=None):
-        '''
+        """
         Download a file from Storage server.
         arguments:
-        @local_filename: string, local name of file 
+        @local_filename: string, local name of file
         @remote_file_id: string, file_id of file that is on storage server
         @offset: long
         @downbytes: long
@@ -347,7 +365,7 @@ class Fdfs_client(object):
             'Download size'   : downloaded_size,
             'Storage IP'      : storage_ip
         }
-        '''
+        """
         tmp = split_remote_fileid(remote_file_id)
         if not tmp:
             raise DataError('[-] Error: remote_file_id is invalid.(in download file)')
@@ -356,14 +374,14 @@ class Fdfs_client(object):
             file_offset = int(offset)
         if not down_bytes:
             download_bytes = int(down_bytes)
-        tc = Tracker_client(self.tracker_pool)
+        tc = Tracker_client(self.tracker_pool, self.trackers)
         store_serv = tc.tracker_query_storage_fetch(group_name, remote_filename)
         store = Storage_client(store_serv.ip_addr, store_serv.port, self.timeout)
         return store.storage_download_to_file(tc, store_serv, local_filename, file_offset, download_bytes,
                                               remote_filename, file_crypt)
 
     def download_to_buffer(self, remote_file_id, offset=0, down_bytes=0):
-        '''
+        """
         Download a file from Storage server and store in buffer.
         arguments:
         @remote_file_id: string, file_id of file that is on storage server
@@ -375,7 +393,7 @@ class Fdfs_client(object):
             'Download size'   : downloaded_size,
             'Storage IP'      : storage_ip
         }
-        '''
+        """
         tmp = split_remote_fileid(remote_file_id)
         if not tmp:
             raise DataError('[-] Error: remote_file_id is invalid.(in download file)')
@@ -384,7 +402,7 @@ class Fdfs_client(object):
             file_offset = int(offset)
         if not down_bytes:
             download_bytes = int(down_bytes)
-        tc = Tracker_client(self.tracker_pool)
+        tc = Tracker_client(self.tracker_pool, self.trackers)
         store_serv = tc.tracker_query_storage_fetch(group_name, remote_filename)
         store = Storage_client(store_serv.ip_addr, store_serv.port, self.timeout)
         file_buffer = None
@@ -392,17 +410,17 @@ class Fdfs_client(object):
                                                 remote_filename)
 
     def list_one_group(self, group_name):
-        '''
+        """
         List one group information.
         arguments:
         @group_name: string, group name will be list
         @return Group_info,  instance
-        '''
-        tc = Tracker_client(self.tracker_pool)
+        """
+        tc = Tracker_client(self.tracker_pool, self.trackers)
         return tc.tracker_list_one_group(group_name)
 
     def list_servers(self, group_name, storage_ip=None):
-        '''
+        """
         List all storage servers information in a group
         arguments:
         @group_name: string
@@ -410,39 +428,39 @@ class Fdfs_client(object):
             'Group name' : group_name,
             'Servers'    : server list,
         }
-        '''
-        tc = Tracker_client(self.tracker_pool)
+        """
+        tc = Tracker_client(self.tracker_pool, self.trackers)
         return tc.tracker_list_servers(group_name, storage_ip)
 
     def list_all_groups(self):
-        '''
+        """
         List all group information.
         @return dictionary {
             'Groups count' : group_count,
             'Groups'       : list of groups
         }
-        '''
-        tc = Tracker_client(self.tracker_pool)
+        """
+        tc = Tracker_client(self.tracker_pool, self.trackers)
         return tc.tracker_list_all_groups()
 
     def get_meta_data(self, remote_file_id):
-        '''
+        """
         Get meta data of remote file.
         arguments:
         @remote_fileid: string, remote file id
         @return dictionary, meta data
-        '''
+        """
         tmp = split_remote_fileid(remote_file_id)
         if not tmp:
             raise DataError('[-] Error: remote_file_id is invalid.(in get meta data)')
         group_name, remote_filename = tmp
-        tc = Tracker_client(self.tracker_pool)
+        tc = Tracker_client(self.tracker_pool, self.trackers)
         store_serv = tc.tracker_query_storage_update(group_name, remote_filename)
         store = Storage_client(store_serv.ip_addr, store_serv.port, self.timeout)
         return store.storage_get_metadata(tc, store_serv, remote_filename)
 
     def set_meta_data(self, remote_file_id, meta_dict, op_flag=STORAGE_SET_METADATA_FLAG_OVERWRITE):
-        '''
+        """
         Set meta data of remote file.
         arguments:
         @remote_file_id: string
@@ -452,12 +470,12 @@ class Fdfs_client(object):
             'Status'     : status,
             'Storage IP' : storage_ip
         }
-        '''
+        """
         tmp = split_remote_fileid(remote_file_id)
         if not tmp:
             raise DataError('[-] Error: remote_file_id is invalid.(in set meta data)')
         group_name, remote_filename = tmp
-        tc = Tracker_client(self.tracker_pool)
+        tc = Tracker_client(self.tracker_pool, self.trackers)
         try:
             store_serv = tc.tracker_query_storage_update(group_name, remote_filename)
             store = Storage_client(store_serv.ip_addr, store_serv.port, self.timeout)
@@ -467,7 +485,7 @@ class Fdfs_client(object):
         # if status == 2:
         #    raise DataError('[-] Error: remote file %s is not exist.' % remote_file_id)
         if status != 0:
-            raise DataError('[-] Error: %d, %s' % (th.status, os.strerror(th.status)))
+            raise DataError('[-] Error: %d, %s' % (status, os.strerror(status)))
         ret_dict = {}
         ret_dict['Status'] = 'Set meta data success.'
         ret_dict['Storage IP'] = store_serv.ip_addr
@@ -481,7 +499,7 @@ class Fdfs_client(object):
         if not tmp:
             raise DataError('[-] Error: remote_file_id is invalid.(append)')
         group_name, appended_filename = tmp
-        tc = Tracker_client(self.tracker_pool)
+        tc = Tracker_client(self.tracker_pool, self.trackers)
         store_serv = tc.tracker_query_storage_update(group_name, appended_filename)
         store = Storage_client(store_serv.ip_addr, store_serv.port, self.timeout)
         return store.storage_append_by_filename(tc, store_serv, local_filename, appended_filename)
@@ -494,7 +512,7 @@ class Fdfs_client(object):
         if not tmp:
             raise DataError('[-] Error: remote_file_id is invalid.(append)')
         group_name, appended_filename = tmp
-        tc = Tracker_client(self.tracker_pool)
+        tc = Tracker_client(self.tracker_pool, self.trackers)
         store_serv = tc.tracker_query_storage_update(group_name, appended_filename)
         store = Storage_client(store_serv.ip_addr, store_serv.port, self.timeout)
         return store.storage_append_by_file(tc, store_serv, local_filename, appended_filename)
@@ -506,7 +524,7 @@ class Fdfs_client(object):
         if not tmp:
             raise DataError('[-] Error: remote_file_id is invalid.(append)')
         group_name, appended_filename = tmp
-        tc = Tracker_client(self.tracker_pool)
+        tc = Tracker_client(self.tracker_pool, self.trackers)
         store_serv = tc.tracker_query_storage_update(group_name, appended_filename)
         store = Storage_client(store_serv.ip_addr, store_serv.port, self.timeout)
         return store.storage_append_by_buffer(tc, store_serv, file_buffer, appended_filename)
@@ -527,13 +545,13 @@ class Fdfs_client(object):
         if not tmp:
             raise DataError('[-] Error: appender_fileid is invalid.(truncate)')
         group_name, appender_filename = tmp
-        tc = Tracker_client(self.tracker_pool)
+        tc = Tracker_client(self.tracker_pool, self.trackers)
         store_serv = tc.tracker_query_storage_update(group_name, appender_filename)
         store = Storage_client(store_serv.ip_addr, store_serv.port, self.timeout)
         return store.storage_truncate_file(tc, store_serv, trunc_filesize, appender_filename)
 
     def modify_by_filename(self, filename, appender_fileid, offset=0):
-        '''
+        """
         Modify a file in Storage server by file.
         arguments:
         @filename: string, local file name
@@ -543,7 +561,7 @@ class Fdfs_client(object):
             'Status'     : 'Modify successed.',
             'Storage IP' : storage_ip
         }
-        '''
+        """
         isfile, errmsg = fdfs_check_file(filename)
         if not isfile:
             raise DataError(errmsg + '(modify)')
@@ -556,13 +574,13 @@ class Fdfs_client(object):
             file_offset = int(offset)
         else:
             file_offset = 0
-        tc = Tracker_client(self.tracker_pool)
+        tc = Tracker_client(self.tracker_pool, self.trackers)
         store_serv = tc.tracker_query_storage_update(group_name, appender_filename)
         store = Storage_client(store_serv.ip_addr, store_serv.port, self.timeout)
         return store.storage_modify_by_filename(tc, store_serv, filename, file_offset, filesize, appender_filename)
 
     def modify_by_file(self, filename, appender_fileid, offset=0):
-        '''
+        """
         Modify a file in Storage server by file.
         arguments:
         @filename: string, local file name
@@ -572,7 +590,7 @@ class Fdfs_client(object):
             'Status'     : 'Modify successed.',
             'Storage IP' : storage_ip
         }
-        '''
+        """
         isfile, errmsg = fdfs_check_file(filename)
         if not isfile:
             raise DataError(errmsg + '(modify)')
@@ -585,13 +603,13 @@ class Fdfs_client(object):
             file_offset = int(offset)
         else:
             file_offset = 0
-        tc = Tracker_client(self.tracker_pool)
+        tc = Tracker_client(self.tracker_pool, self.trackers)
         store_serv = tc.tracker_query_storage_update(group_name, appender_filename)
         store = Storage_client(store_serv.ip_addr, store_serv.port, self.timeout)
         return store.storage_modify_by_file(tc, store_serv, filename, file_offset, filesize, appender_filename)
 
     def modify_by_buffer(self, filebuffer, appender_fileid, offset=0):
-        '''
+        """
         Modify a file in Storage server by buffer.
         arguments:
         @filebuffer: string, file buffer
@@ -601,7 +619,7 @@ class Fdfs_client(object):
             'Status'     : 'Modify successed.',
             'Storage IP' : storage_ip
         }
-        '''
+        """
         if not filebuffer:
             raise DataError('[-] Error: filebuffer can not be null.(modify)')
         filesize = len(filebuffer)
@@ -613,7 +631,7 @@ class Fdfs_client(object):
             file_offset = int(offset)
         else:
             file_offset = 0
-        tc = Tracker_client(self.tracker_pool)
+        tc = Tracker_client(self.tracker_pool, self.trackers)
         store_serv = tc.tracker_query_storage_update(group_name, appender_filename)
         store = Storage_client(store_serv.ip_addr, store_serv.port, self.timeout)
         return store.storage_modify_by_buffer(tc, store_serv, filebuffer, file_offset, filesize, appender_filename)
